import React, { useState, useRef, useCallback, useEffect } from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var DragHandler = function (props) {
    var dir = props.dir, onStart = props.onStart, onEnd = props.onEnd, onUpdate = props.onUpdate, className = props.className, style = props.style, children = props.children;
    var _a = useState(false), isDragging = _a[0], setIsDragging = _a[1];
    var listenersRef = useRef(null);
    var handleMouseMove = useCallback(function (e) {
        onUpdate(e);
    }, [onUpdate]);
    var cleanMouseListeners = useCallback(function () {
        var oldRef = listenersRef.current;
        if (oldRef) {
            window.removeEventListener('mousemove', oldRef.handleMouseMove);
            window.removeEventListener('mouseup', oldRef.handleMouseUp);
        }
    }, []);
    var handleMouseUp = useCallback(function (e) {
        setIsDragging(false);
        cleanMouseListeners();
        onEnd(e);
    }, [cleanMouseListeners, onEnd]);
    var handleMouseDown = useCallback(function (e) {
        setIsDragging(true);
        cleanMouseListeners();
        listenersRef.current = {
            handleMouseMove: handleMouseMove,
            handleMouseUp: handleMouseUp,
        };
        window.addEventListener('mousemove', handleMouseMove);
        window.addEventListener('mouseup', handleMouseUp);
        onStart(e);
    }, [cleanMouseListeners, handleMouseMove, handleMouseUp, onStart]);
    useEffect(function () {
        return function () {
            cleanMouseListeners();
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return (React.createElement("div", { onMouseDown: handleMouseDown, className: className, style: __assign({ cursor: dir + "-resize" }, style) },
        isDragging && (React.createElement("style", null, "\n        * {\n          cursor: " + dir + "-resize !important;\n          -webkit-user-select: none !important;\n        }\n        ")),
        children));
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
var isNil = function (v) { return v === null || v === undefined; };
var getContainerMeta = function (_a) {
    var border = _a.border;
    var wh;
    var xy;
    var sn;
    if (/^(left|right)$/.test(border)) {
        wh = 'width';
        xy = 'clientX';
        sn = border === 'right' ? 1 : -1;
    }
    else {
        wh = 'height';
        xy = 'clientY';
        sn = border === 'bottom' ? 1 : -1;
    }
    return { wh: wh, xy: xy, sn: sn };
};
var getContainerInfo = function (_a) {
    var style = _a.style, containerMeta = _a.containerMeta, diffCoord = _a.diffCoord, oldSize = _a.oldSize;
    var wh = containerMeta.wh, sn = containerMeta.sn;
    var retStyle = {};
    if (oldSize != null) {
        retStyle[wh] = oldSize + diffCoord * sn;
    }
    retStyle = __assign(__assign({}, style), retStyle);
    return { style: retStyle };
};
var getHandlerInfo = function (_a) {
    var border = _a.border, handlerWidth = _a.handlerWidth, handlerOffset = _a.handlerOffset, handlerStyle = _a.handlerStyle;
    var dir;
    var style = {};
    if (/^(left|right)$/.test(border)) {
        dir = 'ew';
        style.width = handlerWidth;
        style.top = 0;
        style.bottom = 0;
    }
    else {
        dir = 'ns';
        style.height = handlerWidth;
        style.left = 0;
        style.right = 0;
    }
    style[border] = handlerOffset;
    style = __assign(__assign({}, style), handlerStyle);
    return { dir: dir, style: style };
};

var DragSizing = function (props) {
    var border = props.border, onStart = props.onStart, onEnd = props.onEnd, onUpdate = props.onUpdate, id = props.id, className = props.className, style = props.style, handlerClassName = props.handlerClassName, _handlerStyle = props.handlerStyle, _handlerWidth = props.handlerWidth, _handlerOffset = props.handlerOffset, _handlerZIndex = props.handlerZIndex, children = props.children;
    var handlerWidth = isNil(_handlerWidth) ? 16 : _handlerWidth;
    var handlerOffset = (isNil(_handlerOffset)
        ? -handlerWidth / 2
        : _handlerOffset);
    var handlerZIndex = (isNil(_handlerZIndex) ? 10 : _handlerZIndex);
    var _a = useState(0), diffCoord = _a[0], setDiffCoord = _a[1];
    var _b = useState(null), oldSize = _b[0], setOldSize = _b[1];
    var oldCoordRef = useRef(null);
    var boxRef = useRef(null);
    var containerMeta = getContainerMeta({ border: border });
    var containerStyle = getContainerInfo({
        style: style,
        containerMeta: containerMeta,
        diffCoord: diffCoord,
        oldSize: oldSize,
    }).style;
    var _c = getHandlerInfo({
        border: border,
        handlerWidth: handlerWidth,
        handlerOffset: handlerOffset,
        handlerStyle: _handlerStyle,
    }), dir = _c.dir, handlerStyle = _c.style;
    var handleStart = useCallback(function (e) {
        var wh = containerMeta.wh, xy = containerMeta.xy;
        var el = boxRef.current;
        if (!el)
            return;
        var px = window.getComputedStyle(el)[wh];
        setDiffCoord(0);
        setOldSize(parseInt(px, 10));
        oldCoordRef.current = e[xy];
        if (onStart)
            onStart(e);
    }, [containerMeta, onStart]);
    var handleEnd = useCallback(function (e) {
        if (onEnd)
            onEnd(e);
    }, [onEnd]);
    var handleUpdate = useCallback(function (e) {
        var xy = containerMeta.xy;
        if (oldCoordRef.current === null)
            return;
        setDiffCoord(e[xy] - oldCoordRef.current);
        if (onUpdate)
            onUpdate(e);
    }, [containerMeta, onUpdate]);
    return (React.createElement("div", { ref: boxRef, id: id, className: className, style: __assign({ position: 'relative' }, containerStyle) },
        React.createElement(DragHandler, { dir: dir, className: handlerClassName, style: __assign({ position: 'absolute', zIndex: handlerZIndex }, handlerStyle), onStart: handleStart, onEnd: handleEnd, onUpdate: handleUpdate }),
        children));
};

export { DragSizing };
//# sourceMappingURL=index.es.js.map
